/**
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */


package cz.xelfi.quoridor.visidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Move;
import java.awt.GridLayout;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.swing.JPanel;

/** JavaBean class that displays and can manipulate with Quoridor {@link Board}.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public final class BoardPane extends JPanel {
    private final Viewer viewer;
    private final List<BoardListener> listeners = new CopyOnWriteArrayList<BoardListener>();
    
    public BoardPane() {
        setLayout(new GridLayout(1,1));
        viewer = new Viewer();
        viewer.moveListener(this);
        add(viewer);
    }
    
    public final void setBoard(Board b) {
        viewer.setBoard(b);
    }
    
    public final Board getBoard() {
        return viewer.getBoard();
    }

    /** The board can either be in <q>view only</q> mode when showing state 
     * of the {@link Board} during opponent's move, or it can be editable.
     * In editable mode, one shall listen on changes delivered to 
     * BoardListener.
     * 
     * @param editable read only or editable mode?
     */
    public final void setEditable(boolean editable) {
        viewer.enableListeners(editable);
    }
    
    public final void addBoardListener(BoardListener l) {
        listeners.add(l);
    }
    public final void removeBoardListener(BoardListener l) {
        listeners.remove(l);
    }
    
    final void moveHappened(Move m) {
        BoardEvent ev = null;
        for (BoardListener bl : listeners) {
            if (ev == null) {
                ev = new BoardEvent(this, m);
            }
            bl.boardChanged(ev);
        }
    }
}
